package
{
	import flash.display.*;
	import flash.geom.*;
	import flash.ui.*;
	
	public class Level extends Sprite
	{
		public var tiles : TileGrid;
		
		[Embed(source="levels/level1.txt", mimeType="application/octet-stream")]
		public static var level1file : Class;

		[Embed(source="levels/level2.txt", mimeType="application/octet-stream")]
		public static var level2file : Class;
		
		public static const lastLevel : int = 2;

		[Embed(source="images/title.png")]
		public static var titleImage : Class;
		
		[Embed(source="images/congrats.png")]
		public static var congratsImage : Class;
		
		public var entities : Array;
		public var particles : Array;
		
		public var player : Player;
		
		public var levelID : int;
		
		private var entityOverlay : Sprite;
		private var wallOverlay : Sprite;
		private var particleOverlay : Sprite;
		
		public function Level (i : int = 1)
		{
			levelID = i;
			
			if (levelID == 1) {
				addChild(new titleImage());
			} else if (levelID == 2) {
				addChild(new congratsImage());
			}
			
			switch (levelID)
			{
				case 1:
					tiles = new TileGrid(new level1file());
				break;
				case 2:
					tiles = new TileGrid(new level2file());
				break;
			}
			
			addChild(tiles);
			
			entityOverlay = new Sprite();
			addChild(entityOverlay);
			entities = new Array();
			
			wallOverlay = new Sprite();
			addChild(wallOverlay);
			
			particleOverlay = new Sprite();
			addChild(particleOverlay);
			particles = new Array();
			
			Wall.stopped = false;
		}
		
		public function update (dt: int): void
		{
			tiles.update(dt);
			
			for (var i : int = 0; i < entities.length; i++)
			{
				entities[i].update(dt);
			}
			
			for (i = 0; i < particles.length; i++)
			{
				particles[i].update(dt);
			}
			
			particles = particles.filter(removeParticlesFilter);
		}
		
		public function addEntity (entity : DisplayObject) : void
		{
			entities.push(entity);
			
			if (entity is Wall)
			{
				wallOverlay.addChild(entity);
			}
			else
			{
				entityOverlay.addChild(entity);
			}
			
			if (entity is Player)
			{
				player = entity as Player;
			}
		}
		
		private function removeParticlesFilter (element : *, index: int, arr: Array): Boolean
		{
			if (element.dead())
			{
				particleOverlay.removeChild(element);
				
				return false;
			}
			else
			{
				// Element remains in array
				return true;
			}
		}
		
		public function spark (x : Number, y : Number, num : int, lifetime : int = 350) : void
		{
			var s : Spark;
			
			for (var i : int = 0; i < num; i++)
			{
				s = new Spark(x, y, lifetime);
				
				particleOverlay.addChild(s);
				
				particles.push(s);
			}
		}
		
		private static function returnTrue (foo : *) : Boolean
		{
			return true;
		}
		
		public function get (entity: *, testFunc : Function = null) : Array
		{
			if (testFunc == null) { testFunc = returnTrue; }
			
			var rect : Rectangle = entity.getAABB();
			
			var array : Array = new Array();
			
			var ix1 : int = rect.left / Tile.WIDTH;
			var ix2 : int = rect.right / Tile.WIDTH;
			
			var iy1 : int = rect.top / Tile.HEIGHT;
			var iy2 : int = rect.bottom / Tile.HEIGHT;
			
			for (var ix : int = ix1; ix <= ix2; ix++)
			{
				for (var iy : int = iy1; iy <= iy2; iy++)
				{
					var tile : * = tiles.getFromIndex(ix, iy);
					
					if (tile && tile != entity && testFunc(tile))
					{
						array.push(tile);
					}
				}
			}
			
			for (var i : int = 0; i < entities.length; i++)
			{
				if (entities[i] != entity
					&& entities[i].getAABB().intersects(rect)
					&& testFunc(entities[i])
					)
				{
					array.push(entities[i]);
				}
			}
			
			return array;
		}
	}
}

