package
{
	import flash.display.*;
	import flash.geom.*;
	import flash.ui.*;
	
	public class Player extends Sprite
	{
		[Embed(source="images/robot.png")]
		public static var robotSrc:Class;
		
		[Embed(source="images/robot-dead.png")]
		public static var robotDeadSrc:Class;
		
		[Embed(source="images/spring.png")]
		public static var springSrc:Class;
		
		public static var facing : int = -1;
		
		private var spring : Bitmap;
		private var robot : Bitmap;
		private var robotDead : Bitmap;
		
		private var springI : int = 0;
		
		private static const springLengths : Array = new Array(
			//0, 4, 4, 3, 2, 1, 4, 6, 8, 5, 8, 5, 8, 5, 8, 5, 8, 5, 5, 5, 5, 4, 3, 2, 1, 0
			  0, 4, 3, 2, 2, 1, 4, 6, 8, 5, 8, 5, 8, 5, 8, 5, 8, 5, 5, 5, 5, 4, 3, 2, 1, 0
		);
		
		private var vy : Number = 0;
		
		private var standing : Boolean = false;
		
		public static var dead : Boolean = false;
		
		public function Player (_x : Number, _y : Number)
		{
			spring = new springSrc();
			
			spring.x = -16;
			
			addChild(spring);
			
			robot = new robotSrc();
			
			robot.x = -16;
			
			addChild(robot);
			
			robotDead = new robotDeadSrc();
			
			robotDead.x = -16;
			robotDead.visible = false;
			
			addChild(robotDead);
			
			x = _x + 16;
			y = _y;
			
			dead = false;
			facing = -1;
		}
		
		public function getAABB () : Rectangle
		{
			return new Rectangle(x - 8, y + 4, 16, 28);
		}
		
		private function isSolid (obj : *) : Boolean
		{
			return (obj is Wall || obj is Button) ? false : true;
		}
		
		private function isWall (obj : *) : Boolean
		{
			return (obj is Wall);
		}
		
		public function checkX () : void
		{
			var playerRect : Rectangle = getAABB();
			
			var entities : Array = Main.level.get(this, isSolid);
			
			var overlapLeft : Number = 0;
			var overlapRight : Number = 0;
			
			for (var i : int = 0; i < entities.length; i++)
			{
				var otherRect : Rectangle = entities[i].getAABB();
				
				if (playerRect.bottom <= otherRect.top || otherRect.bottom <= playerRect.top) { continue; }
				
				var a : Number = playerRect.right - otherRect.left;
				var b : Number = otherRect.right - playerRect.left;
				
				if (a < b)
				{
					overlapRight = Math.max(a, overlapRight);
				} else {
					overlapLeft = Math.max(b, overlapLeft);
				}
			}
			
			if (overlapLeft > overlapRight) {
				x += overlapLeft;
			} else {
				x -= overlapRight;
			}
		}
		
		public function checkY () : void
		{
			var playerRect : Rectangle = getAABB();
			
			var entities : Array = Main.level.get(this, isSolid);
			
			var overlapTop : Number = 0;
			var overlapBottom : Number = 0;
			
			for (var i : int = 0; i < entities.length; i++)
			{
				var otherRect : Rectangle = entities[i].getAABB();
				
				if (playerRect.left >= otherRect.right || otherRect.left >= playerRect.right) { continue; }
				
				var a : Number = playerRect.bottom - otherRect.top;
				var b : Number = otherRect.bottom - playerRect.top;
				
				if (a < b)
				{
					overlapBottom = Math.max(a, overlapBottom);
				} else {
					overlapTop = Math.max(b, overlapTop);
				}
			}
			
			if (overlapTop > overlapBottom) { // hit his head
				y += overlapTop;
				vy = Math.max(vy, 0);
				//springI = springLengths.length - 3;
				Main.level.spark(x, y + 4, 1);
				AudioControl.playHitHeadSound();
			} else if (overlapBottom > 0) { // standing on floor
				y -= overlapBottom;
				vy = Math.min(vy, 0);
				standing = true;
			}
		}
		
		public function update (dt: int): void
		{
			if (! dead && ! Wall.stopped)
			{
				var dir : int = 0;
				if (Input.keyPressed(Keyboard.RIGHT)) { dir += 1; }
				if (Input.keyPressed(Keyboard.LEFT)) { dir -= 1; }
				
				if (dir != 0) {
					facing = dir;
				}
				
				robot.scaleX = facing;
				robot.x = facing * -16;
				
				x += dir * 0.15 * dt;
				
				checkX();
			}
			
			var a : Number = 0;
			
			if (Input.keyPressed(Keyboard.UP) && standing && !dead)
			{
				vy = -0.45;
				standing = false;
				springI = 1;
				AudioControl.playJumpSound();
			}
			
			a = 0.001;
			vy += a * dt;
			
			y += vy * dt - 0.5 * a * dt * dt;
			
			standing = false;
			
			checkY();
			
			if (standing)
			{
				springI = 0;
			}
			
			if (springI > 0)
			{
				spring.y = 28 - 8 + springLengths[springI];
				springI++;
				
				if (springI >= springLengths.length)
				{
					springI = 0;
				}
			}
			else
			{
				spring.y = 28 - 8;
			}
			
			if (!dead && Main.level.get(this, isWall).length > 0)
			{
				dead = true;
				AudioControl.playDeathSound();
			}
			
			if (dead)
			{
				if (Math.random() < 0.5)
				{
					Main.level.spark(x, y + 12, Math.random() * 3, 1000)
				}
				
				robotDead.visible = true;
				robot.visible = false;
				spring.visible = false;
				
				robotDead.scaleX = facing;
				robotDead.x = facing * -16;
			}
		}
	}
}

