package
{
	import flash.display.*;
	import flash.utils.*;
	
	public class TileGrid extends Sprite
	{
		public static const WIDTH : int = 20;
		public static const HEIGHT : int = 15;
		
		private var tiles : Array;
		
		public function TileGrid (input : ByteArray)
		{
			var x : int, y : int;
			
			tiles = new Array();
			
			for (x = 0; x < WIDTH; x++)
			{
				tiles[x] = new Array();
			}
			
			if (input) {
				var text : String = input.toString();
				
				var lines : Array = text.split("\n");
				
				for (y = 0; y < lines.length; y++)
				{
					var tilesOnLine : Array = lines[y].split(/\s/);
				
					for (x = 0; x < tilesOnLine.length; x++)
					{
						var tileID : String = tilesOnLine[x];
						
						if (tileID == "") { continue; }
						
						put(Tile.create(tileID), x, y);
					}
				}
			}
		}
		
		public function update (dt : int) : void
		{
			for (var y : int = 0; y < HEIGHT; y++)
			{
				for (var x : int = 0; x < WIDTH; x++)
				{
					var tile : Tile = tiles[x][y];
					
					if (tile) {
						tile.update(dt);
					}
				}
			}
		}
		
		public function getFromIndex (x : int, y : int) : Tile
		{
			if (x < 0 || x >= WIDTH || y < 0 || y > HEIGHT) { return null; }
			
			return tiles[x][y];
		}
		
		public function getFromPixel (x : Number, y : Number) : Tile
		{
			var ix : int = x / Tile.WIDTH;
			var iy : int = y / Tile.HEIGHT;
			
			return tiles[ix][iy];
		}
		
		public function removeAtIndex (x : int, y : int) : void
		{
			if (tiles[x][y])
			{
				removeChild(tiles[x][y]);
				tiles[x][y] = null;
			}
		}
		
		public function put (obj : Tile, x : int, y : int) : void
		{
			if (tiles[x][y])
			{
				removeChild(tiles[x][y]);
			}
			
			tiles[x][y] = obj;
			
			if (obj)
			{
				obj.x = x * Tile.WIDTH;
				obj.y = y * Tile.HEIGHT;
				addChild(obj);
			}
		}
	}
}

